/*
    SignalGenerator.pde
    Copyright (C) 2011
    Authors: Arun I, Hari Prasad Gokul R, Prabhu P, Sravan M Baddepudi, Sudhakar M K, Suresh Kannan M
    email: theroarofthedragon@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

//class to generate various discrete time input signals

class SignalGenerator
{
  SignalGenerator() {}
  
  DTDataPair unitImpulseSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      if (output.n[i]==0) output.x[i]=1;
      else output.x[i]=0;
    }
    return(output);
  }
  
  DTDataPair unitStepSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      if (output.n[i]>=0) output.x[i]=1;
      else output.x[i]=0;
    }
    return(output);
  }
  
  DTDataPair unitRampSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      if (output.n[i]>0) output.x[i]=output.n[i];
      else output.x[i]=0;
    }
    return(output);
  }
  
  DTDataPair signumSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      if (output.n[i]>0) output.x[i]=1;
      else if (output.n[i]<0) output.x[i]=-1;
      else output.x[i]=0;
    }
    return(output);
  }
  
  DTDataPair sineSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      output.x[i]=sin(output.n[i]);
    }
    return(output);
  }
  
  DTDataPair squareSequence(int nMin,int nMax,int period)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      if (output.n[i]>=0) output.x[i]=abs(output.n[i]%period)<period/2?1:-1;
      else output.x[i]=abs((output.n[i]+1)%period)<period/2?-1:1;
    }
    return(output);
  }
  
  DTDataPair sincSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      if (output.n[i]==0) output.x[i]=1;
      else output.x[i]=sin(output.n[i]*PI)/(output.n[i]*PI);
    }
    return(output);
  }
  
  DTDataPair expSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      output.x[i]=exp(output.n[i]);
    }
    return(output);
  }
  
  DTDataPair logSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      output.x[i]=log(output.n[i]);
    }
    return(output);
  }
  
  DTDataPair absoluteSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      output.x[i]=abs(output.n[i]);
    }
    return(output);
  }
}
