import processing.core.*; 
import processing.xml.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class SignalOperations800x600 extends PApplet {

/*
    SignalOperations800x600.pde
    Copyright (C) 2011
    Authors: Arun I, Hari Prasad Gokul R, Prabhu P, Sravan M Baddepudi, Sudhakar M K, Suresh Kannan M
    email: theroarofthedragon@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

/* Program to illustrate the concepts of various fundamental signal operations
 Done as part of Signals and Systems Assignment, fourth semester, BE ECE
 */

int SIZE=100;
int START=-100;
float scalingFactor;

PFont f;
Slider scaler,shifter;
DTDataPair input,output;
Operations operator;
PlotAxes inputAxes,outputAxes;
SignalGenerator agilent;
Button fold,currentSignal;
Button impulse,step,sine,square,exponential,logarithm,absolute;

public void setup()
{
  //initialize font
  f=loadFont("WenQuanYiMicroHeiMono-15.vlw");

  //initialize sketch
  size(800,600);
  background(255);
  smooth();

  //initialize controls
  scaler=new Slider(630,100,150,30,-10,9);
  shifter=new Slider(630,200,150,30,-10,10);
  fold=new Button(650,300,50,30,"Fold",f);

  //initialize signal buttons
  impulse=new Button(530,50,80,30,"Impulse",f);
  step=new Button(530,100,80,30,"Step",f);
  sine=new Button(530,150,80,30,"Sine",f);
  square=new Button(530,200,80,30,"Square",f);
  exponential=new Button(530,250,80,30,"Exp",f);
  logarithm=new Button(530,300,80,30,"Log",f);
  absolute=new Button(530,350,80,30,"Abs",f);

  //initialize plots, signal generator and operator
  inputAxes=new PlotAxes(100,30,400,200,f);
  outputAxes=new PlotAxes(100,330,400,200,f);
  agilent=new SignalGenerator();
  operator=new Operations();

  //set initial signal
  currentSignal=impulse;
  input=agilent.unitImpulseSequence(-10,10);
  output=input;
  inputAxes.stem(input);
  outputAxes.stem(input);
  labelPlots(1,0);
}

public void draw()
{
}

public void mousePressed()
{
  //check for press of fold button
  if (fold.pressed())
  {
    fold.update();
    scalingFactor=PApplet.parseInt(scaler.curValue)<=0?PApplet.parseFloat(1)/PApplet.parseFloat(-PApplet.parseInt(scaler.curValue)+1):PApplet.parseFloat(PApplet.parseInt(scaler.curValue));
    scalingFactor=fold.locked==false?scalingFactor:-scalingFactor;
    output=operator.DTOperate(input,scalingFactor,PApplet.parseInt(shifter.curValue));
    currentSignal.update();
    outputAxes.stem(output);
    labelPlots(scalingFactor,PApplet.parseInt(shifter.curValue));
  }

  //check for press of input signal buttons
  if (impulse.pressed())  updateSignal(impulse);
  else if (step.pressed()) updateSignal(step);
  else if (sine.pressed()) updateSignal(sine);
  else if (square.pressed()) updateSignal(square);
  else if (exponential.pressed()) updateSignal(exponential);
  else if (logarithm.pressed()) updateSignal(logarithm);
  else if (absolute.pressed()) updateSignal(absolute);
}

public void mouseDragged()
{
  //update sliders
  scaler.update();
  shifter.update();

  //calculate scaling factor
  scalingFactor=PApplet.parseInt(scaler.curValue)<=0?PApplet.parseFloat(1)/PApplet.parseFloat(-PApplet.parseInt(scaler.curValue)+1):PApplet.parseFloat(PApplet.parseInt(scaler.curValue));
  scalingFactor=fold.locked==false?scalingFactor:-scalingFactor;

  //perform operations on the signal
  output=operator.DTOperate(input,scalingFactor,PApplet.parseInt(shifter.curValue));

  //plot and label the signals
  outputAxes.stem(output);
  labelPlots(scalingFactor,PApplet.parseInt(shifter.curValue));
}

public void updateSignal(Button newSignal)
{
  //unlock previously selected signal
  currentSignal.locked=false;
  currentSignal.update();

  //lock currently selected signal
  currentSignal=newSignal;
  currentSignal.locked=true;

  //generate the input signal
  if (newSignal==impulse)  input=agilent.unitImpulseSequence(-10,10);
  if (newSignal==step)  input=agilent.unitStepSequence(-10,10);
  if (newSignal==sine)  input=agilent.sineSequence(-10,10);
  if (newSignal==square)  input=agilent.squareSequence(-10,10,4);
  if (newSignal==exponential)  input=agilent.expSequence(-10,10);
  if (newSignal==logarithm)  input=agilent.logSequence(1,10);
  if (newSignal==absolute) input=agilent.absoluteSequence(-10,10);
  output=input;
  currentSignal.update();
  inputAxes.stem(input);
  outputAxes.stem(output);

  //reset sliders
  scaler.curValue=0;
  shifter.curValue=0;
  fold.locked=false;
  scaler.drawMe();
  shifter.drawMe();
  fold.update();

  labelPlots(1,0);
}

public void labelPlots(float a,int b)
{
  //clear label area
  fill(255);
  noStroke();
  rect(230,0,150,20);
  rect(230,295,150,20);
  stroke(0);

  //print input label
  fill(0);
  textAlign(CENTER,TOP);
  text("x[n]",300,10);

  //print output label
  String outputLabel="x[";

  if (a<0) outputLabel=outputLabel+"-";

  if (abs(a)==1)  outputLabel=outputLabel+"n";
  else if (abs(a)>1) outputLabel=outputLabel+abs(round(a))+"n";
  else if (abs(a)<1) outputLabel=outputLabel+"n/"+abs(round(1/a));

  if (b==0) outputLabel=outputLabel+"]";
  else if (b>0) outputLabel=outputLabel+"+"+b+"]";
  else if (b<0) outputLabel=outputLabel+b+"]";

  text(outputLabel,300,300);
}

/*
    Button.pde
    Copyright (C) 2011
    Authors: Arun I, Hari Prasad Gokul R, Prabhu P, Sravan M Baddepudi, Sudhakar M K, Suresh Kannan M
    email: theroarofthedragon@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

//class to create a rectangular button

class Button
{
  //Position and dimensions of the button
  int xPos,yPos;
  int widthButton,heightButton;
  
  //Button properties
  String label;
  boolean locked=false;
  
  //Appearance attributes
  int baseColor=color(0);
  int highlightColor=color(125);
  int currentColor=baseColor,textColor=color(255,0,0);
  PFont font;
  
  Button(int xPos_,int yPos_,int widthButton_,int heightButton_,String label_,PFont font_)
  {
    //Storing construction values for later reference
    xPos=xPos_;
    yPos=yPos_;
    widthButton=widthButton_;
    heightButton=heightButton_;
    label=label_;
    font=font_;
    
    //draw button on the sketch
    drawButton();
  }

  public void update() 
  {
    //set the color of the button according to locked status
    if (locked)
      currentColor = highlightColor;
    else
      currentColor = baseColor;
    //redraw button on the sketch
    drawButton();
  }

  public boolean pressed() 
  {
    //function checks if the button has been pressed
    if(over())
    {
      locked = !locked;
      return true;
    } 
    else
      return false;
  }

  public boolean over() 
  {
    //function checks if mouse is over the button
    if (mouseX >= xPos && mouseX <= xPos+widthButton && mouseY >= yPos && mouseY <= yPos+heightButton)
      return true; 
    else
      return false;
  }
  
  public void drawButton() 
  {
    //draw rectangle for the button
    stroke(255);
    fill(currentColor);
    rect(xPos,yPos,widthButton,heightButton);
    
    //print button label
    fill(textColor);
    textAlign(CENTER,CENTER);
    textFont(font,14);
    text(label,xPos+widthButton/2,yPos+heightButton/2);
  }
}
/*
    DTDataPair.pde
    Copyright (C) 2011
    Authors: Arun I, Hari Prasad Gokul R, Prabhu P, Sravan M Baddepudi, Sudhakar M K, Suresh Kannan M
    email: theroarofthedragon@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

//class to hold an 'n' array and an 'x' array to represent a discrete time signal

class DTDataPair
{
  int[] n;
  float[] x;
  
  DTDataPair(int[] n_,float[] x_)
  {
    n=n_;
    x=x_;
  }
}
/*
    Operations.pde
    Copyright (C) 2011
    Authors: Arun I, Hari Prasad Gokul R, Prabhu P, Sravan M Baddepudi, Sudhakar M K, Suresh Kannan M
    email: theroarofthedragon@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

//class to perform time shifting, time scaling and time folding on discrete time signals

class Operations
{
  Operations() {}
  
  public DTDataPair DTOperate(DTDataPair data,float a,float b)
  {
    int[] nOut=new int[0];
    float[] xOut=new float[0];
    float nTemp;
    
    for (int i=0;i<=data.n.length-1;i++)
    {
      nTemp=(data.n[i]-b)/a;
      if (abs(a)>1)
      {
        if ((nTemp%1)==0)
        {
          nOut=append(nOut,PApplet.parseInt(nTemp));
          xOut=append(xOut,data.x[i]);
        }
      }
      else if (abs(a)<1)
      {
        nOut=append(nOut,PApplet.parseInt(nTemp));
        xOut=append(xOut,data.x[i]);
        for (int j=1;j<=round(1/abs(a))-1;j++)
        {
          nOut=append(nOut,PApplet.parseInt(nTemp)+j);
          xOut=append(xOut,0);
        }
      }
      else
      {
        nOut=append(nOut,PApplet.parseInt(nTemp));
        xOut=append(xOut,data.x[i]);
      }
    }
    if (a<0) xOut=reverse(xOut);
    data=new DTDataPair(nOut,xOut);
    return(data);
  }
}
/*
    PlotAxes.pde
    Copyright (C) 2011
    Authors: Arun I, Hari Prasad Gokul R, Prabhu P, Sravan M Baddepudi, Sudhakar M K, Suresh Kannan M
    email: theroarofthedragon@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

//class to plot an array using the stem plot

class PlotAxes
{
  //Position and dimensions of the plot
  int xPos,yPos;
  int widthPlot,heightPlot;
  
  //Data specifications
  float dataMin,dataMax;
  int nMin,nMax;
  
  //Scale and origin of the axes
  float T,scaleY;                    /* T - Processing distance per unit n;    scaleY - data range per unit Processing heightPlot  */
  int originX,originY;
  
  //Appearance attributes
  int backgroundColor,gridColor;
  int plotColor,flowerColor;
  PFont font;
  
  PlotAxes() {}
  
  PlotAxes(int xPos_,int yPos_,int widthPlot_,int heightPlot_,PFont font_)
  {
    //Storing construction values for later reference
    xPos=xPos_;
    yPos=yPos_;
    widthPlot=widthPlot_;
    heightPlot=heightPlot_;
    backgroundColor=color(255);
    gridColor=color(200,200,255);
    plotColor=color(255,0,0);
    flowerColor=color(255,255,0);
    font=font_;
    
    //Drawing the border of the plot on the sketch
    stroke(gridColor);
    fill(backgroundColor);
    rect(xPos,yPos,widthPlot,heightPlot);
  }
  
  public void stem(DTDataPair data)
  {
    //Drawing the border of the plot on the sketch
    noStroke();
    fill(backgroundColor);
    rect(xPos-100,yPos-10,widthPlot+120,heightPlot+40);
    stroke(gridColor);
    rect(xPos,yPos,widthPlot,heightPlot);
    
    //Storing data specifications for later reference
    if (data.x.length==0)  return;
    dataMin=min(data.x);
    dataMax=max(data.x);
    nMin=min(data.n);
    nMax=max(data.n);

    determineAxes();
    calibrateAxes();

    //Plot the given values 
    pushMatrix();
    translate(originX,originY);
    stroke(plotColor);
    float t;
    
    for (int i=0;i<=nMax-nMin;i++)
    {
      t=(nMin+i)*T;
      strokeWeight(2);
      line(t,0,t,-data.x[i]/scaleY);
      fill(flowerColor);
      ellipse(t,-data.x[i]/scaleY,6,6);
      strokeWeight(1);
    }
    popMatrix();
  }
  
  public void determineAxes()
  {
    //Determine X axis scale and origin
    float range=nMax-nMin;
    range= range==0?2:range;
      
    T=PApplet.parseFloat(widthPlot)/range;
    originX=xPos-PApplet.parseInt(nMin*T);
    
    //Determine Y axis scale and origin
    float dataMin_=dataMin<0?dataMin:0;
    float dataMax_=dataMax>0?dataMax:0;
    scaleY=(dataMax_-dataMin_)/heightPlot;
    scaleY=scaleY==0?2:scaleY;
    originY=yPos+heightPlot+PApplet.parseInt(dataMin_/scaleY);
  }
  
  public void calibrateAxes()
  {   
    calibrateXAxis();
    calibrateYAxis();
  }
  
  public void calibrateXAxis()
  {
    //Determine gapN and gapDistance
    int gapN=round(50/T);
    if (gapN==0) gapN=1;
    float gapDistance=gapN*T;
    
    //Calibrate the x axis
    pushMatrix();
    translate(originX,yPos+heightPlot);
    fill(0);
    textFont(font,14);
    textAlign(CENTER);

    for (int i=nMin/gapN;i<=nMax/gapN;i++)
    {
      stroke(0);
      line(i*gapDistance,5,i*gapDistance,0);
      text(i*gapN,i*gapDistance,20);
      stroke(gridColor);
      line(i*gapDistance,0,i*gapDistance,-heightPlot);
    }
    popMatrix();
  }
  
  public void calibrateYAxis()
  {
    //Determine gapData and gapHeight
    float gapData=50*scaleY;
    float gapHeight;
    int exponent=0;
    while (gapData<1)
    {
      gapData=gapData*10;
      exponent--;
    }
    while (gapData>=10)
    {
      gapData=gapData/10;
      exponent++;
    }
    gapData=round(gapData)*pow(10,exponent);
    gapHeight=gapData/scaleY;
    
    //Calibrate the y axis
    pushMatrix();
    translate(xPos,originY);
    fill(0);
    textFont(font,14);
    textAlign(RIGHT);
    
    float dataMin_=dataMin<0?dataMin:0;
    float dataMax_=dataMax>0?dataMax:0;
    for (int i=PApplet.parseInt(dataMin_/gapData);i<=dataMax_/gapData;i++)
    {
      stroke(0);
      line(-5,-i*gapHeight,0,-i*gapHeight);
      text(i*gapData,-8,-i*gapHeight);
      stroke(gridColor);
      line(0,-i*gapHeight,widthPlot,-i*gapHeight);
    }
  
    popMatrix();
  }
}
/*
    SignalGenerator.pde
    Copyright (C) 2011
    Authors: Arun I, Hari Prasad Gokul R, Prabhu P, Sravan M Baddepudi, Sudhakar M K, Suresh Kannan M
    email: theroarofthedragon@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

//class to generate various discrete time input signals

class SignalGenerator
{
  SignalGenerator() {}
  
  public DTDataPair unitImpulseSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      if (output.n[i]==0) output.x[i]=1;
      else output.x[i]=0;
    }
    return(output);
  }
  
  public DTDataPair unitStepSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      if (output.n[i]>=0) output.x[i]=1;
      else output.x[i]=0;
    }
    return(output);
  }
  
  public DTDataPair unitRampSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      if (output.n[i]>0) output.x[i]=output.n[i];
      else output.x[i]=0;
    }
    return(output);
  }
  
  public DTDataPair signumSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      if (output.n[i]>0) output.x[i]=1;
      else if (output.n[i]<0) output.x[i]=-1;
      else output.x[i]=0;
    }
    return(output);
  }
  
  public DTDataPair sineSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      output.x[i]=sin(output.n[i]);
    }
    return(output);
  }
  
  public DTDataPair squareSequence(int nMin,int nMax,int period)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      if (output.n[i]>=0) output.x[i]=abs(output.n[i]%period)<period/2?1:-1;
      else output.x[i]=abs((output.n[i]+1)%period)<period/2?-1:1;
    }
    return(output);
  }
  
  public DTDataPair sincSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      if (output.n[i]==0) output.x[i]=1;
      else output.x[i]=sin(output.n[i]*PI)/(output.n[i]*PI);
    }
    return(output);
  }
  
  public DTDataPair expSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      output.x[i]=exp(output.n[i]);
    }
    return(output);
  }
  
  public DTDataPair logSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      output.x[i]=log(output.n[i]);
    }
    return(output);
  }
  
  public DTDataPair absoluteSequence(int nMin,int nMax)
  {
    int sizeOfArr=nMax-nMin+1;
    DTDataPair output=new DTDataPair(new int[sizeOfArr],new float[sizeOfArr]);
    for (int i=0;i<=sizeOfArr-1;i++)
    {
      output.n[i]=i+nMin;
      output.x[i]=abs(output.n[i]);
    }
    return(output);
  }
}
/*
    Slider.pde
    Copyright (C) 2011
    Authors: Arun I, Hari Prasad Gokul R, Prabhu P, Sravan M Baddepudi, Sudhakar M K, Suresh Kannan M
    email: theroarofthedragon@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

//class to create a linear slider

class Slider
{
  //Position and dimensions of the slider
  int xPos,yPos;
  int widthSlider,heightSlider;
  int widthKnob;
  
  //Values
  int minValue,maxValue;
  float curValue;
  int range;
  
  Slider(int xPos_,int yPos_,int widthSlider_,int heightSlider_,int minValue_,int maxValue_)
  {
    xPos=xPos_;
    yPos=yPos_;
    widthSlider=widthSlider_;
    heightSlider=heightSlider_;
    minValue=minValue_;
    maxValue=maxValue_;
    curValue=0;
    widthKnob=20;
    range=maxValue-minValue;
    
    drawMe();
  }
  
  public void drawMe()
  {
    pushMatrix();
    translate(xPos,yPos);
    
    fill(255);
    stroke(255);
    strokeWeight(1);
    rect(-widthKnob/2,-heightSlider/2,widthSlider+widthKnob,heightSlider);
    
    stroke(0);
    line(0,0,widthSlider,0);
    float knobPos=map(curValue,minValue,maxValue,0,widthSlider);
    rect(knobPos-widthKnob/2,-heightSlider/2,widthKnob,heightSlider);
    
    popMatrix();
  }
  
  public void update()
  {
    if (overKnob())
    {
      curValue=curValue+(mouseX-pmouseX)*0.25f;
      curValue=curValue>maxValue?maxValue:curValue;
      curValue=curValue<minValue?minValue:curValue;
      drawMe();
    }
  }
  
  public boolean overKnob()
  {
    int knobPos=(int)map(curValue,minValue,maxValue,0,widthSlider);
    boolean over=(mouseX>xPos+knobPos-widthKnob/2 && mouseX<xPos+knobPos+widthKnob/2 && mouseY<yPos+heightSlider/2 && mouseY>yPos-heightSlider/2);
    return(over);
  }
}
  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#DFDFDF", "SignalOperations800x600" });
  }
}
