/*
    SignalOperations800x600.pde
    Copyright (C) 2011
    Authors: Arun I, Hari Prasad Gokul R, Prabhu P, Sravan M Baddepudi, Sudhakar M K, Suresh Kannan M
    email: theroarofthedragon@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

/* Program to illustrate the concepts of various fundamental signal operations
 Done as part of Signals and Systems Assignment, fourth semester, BE ECE
 */

int SIZE=100;
int START=-100;
float scalingFactor;

PFont f;
Slider scaler,shifter;
DTDataPair input,output;
Operations operator;
PlotAxes inputAxes,outputAxes;
SignalGenerator agilent;
Button fold,currentSignal;
Button impulse,step,sine,square,exponential,logarithm,absolute;

void setup()
{
  //initialize font
  f=loadFont("WenQuanYiMicroHeiMono-15.vlw");

  //initialize sketch
  size(800,600);
  background(255);
  smooth();

  //initialize controls
  scaler=new Slider(630,100,150,30,-10,9);
  shifter=new Slider(630,200,150,30,-10,10);
  fold=new Button(650,300,50,30,"Fold",f);

  //initialize signal buttons
  impulse=new Button(530,50,80,30,"Impulse",f);
  step=new Button(530,100,80,30,"Step",f);
  sine=new Button(530,150,80,30,"Sine",f);
  square=new Button(530,200,80,30,"Square",f);
  exponential=new Button(530,250,80,30,"Exp",f);
  logarithm=new Button(530,300,80,30,"Log",f);
  absolute=new Button(530,350,80,30,"Abs",f);

  //initialize plots, signal generator and operator
  inputAxes=new PlotAxes(100,30,400,200,f);
  outputAxes=new PlotAxes(100,330,400,200,f);
  agilent=new SignalGenerator();
  operator=new Operations();

  //set initial signal
  currentSignal=impulse;
  input=agilent.unitImpulseSequence(-10,10);
  output=input;
  inputAxes.stem(input);
  outputAxes.stem(input);
  labelPlots(1,0);
}

void draw()
{
}

void mousePressed()
{
  //check for press of fold button
  if (fold.pressed())
  {
    fold.update();
    scalingFactor=int(scaler.curValue)<=0?float(1)/float(-int(scaler.curValue)+1):float(int(scaler.curValue));
    scalingFactor=fold.locked==false?scalingFactor:-scalingFactor;
    output=operator.DTOperate(input,scalingFactor,int(shifter.curValue));
    currentSignal.update();
    outputAxes.stem(output);
    labelPlots(scalingFactor,int(shifter.curValue));
  }

  //check for press of input signal buttons
  if (impulse.pressed())  updateSignal(impulse);
  else if (step.pressed()) updateSignal(step);
  else if (sine.pressed()) updateSignal(sine);
  else if (square.pressed()) updateSignal(square);
  else if (exponential.pressed()) updateSignal(exponential);
  else if (logarithm.pressed()) updateSignal(logarithm);
  else if (absolute.pressed()) updateSignal(absolute);
}

void mouseDragged()
{
  //update sliders
  scaler.update();
  shifter.update();

  //calculate scaling factor
  scalingFactor=int(scaler.curValue)<=0?float(1)/float(-int(scaler.curValue)+1):float(int(scaler.curValue));
  scalingFactor=fold.locked==false?scalingFactor:-scalingFactor;

  //perform operations on the signal
  output=operator.DTOperate(input,scalingFactor,int(shifter.curValue));

  //plot and label the signals
  outputAxes.stem(output);
  labelPlots(scalingFactor,int(shifter.curValue));
}

void updateSignal(Button newSignal)
{
  //unlock previously selected signal
  currentSignal.locked=false;
  currentSignal.update();

  //lock currently selected signal
  currentSignal=newSignal;
  currentSignal.locked=true;

  //generate the input signal
  if (newSignal==impulse)  input=agilent.unitImpulseSequence(-10,10);
  if (newSignal==step)  input=agilent.unitStepSequence(-10,10);
  if (newSignal==sine)  input=agilent.sineSequence(-10,10);
  if (newSignal==square)  input=agilent.squareSequence(-10,10,4);
  if (newSignal==exponential)  input=agilent.expSequence(-10,10);
  if (newSignal==logarithm)  input=agilent.logSequence(1,10);
  if (newSignal==absolute) input=agilent.absoluteSequence(-10,10);
  output=input;
  currentSignal.update();
  inputAxes.stem(input);
  outputAxes.stem(output);

  //reset sliders
  scaler.curValue=0;
  shifter.curValue=0;
  fold.locked=false;
  scaler.drawMe();
  shifter.drawMe();
  fold.update();

  labelPlots(1,0);
}

void labelPlots(float a,int b)
{
  //clear label area
  fill(255);
  noStroke();
  rect(230,0,150,20);
  rect(230,295,150,20);
  stroke(0);

  //print input label
  fill(0);
  textAlign(CENTER,TOP);
  text("x[n]",300,10);

  //print output label
  String outputLabel="x[";

  if (a<0) outputLabel=outputLabel+"-";

  if (abs(a)==1)  outputLabel=outputLabel+"n";
  else if (abs(a)>1) outputLabel=outputLabel+abs(round(a))+"n";
  else if (abs(a)<1) outputLabel=outputLabel+"n/"+abs(round(1/a));

  if (b==0) outputLabel=outputLabel+"]";
  else if (b>0) outputLabel=outputLabel+"+"+b+"]";
  else if (b<0) outputLabel=outputLabel+b+"]";

  text(outputLabel,300,300);
}

